@@ -23,8 +23,13 @@
 extern "C" {
 #endif
 
+#include <stddef.h>
+#include <stdint.h>
+
 typedef void * opencc_t;
 
+typedef uint32_t ucs4_t;
+
 typedef enum
 {
 	OPENCC_ERROR_VOID,
@@ -40,12 +45,6 @@ typedef enum
 	OPENCC_DICTIONARY_TYPE_DATRIE,
 } opencc_dictionary_type;
 
-typedef struct
-{
-	opencc_dictionary_type dict_type;
-	char * file_name;
-} opencc_dictionary;
-
 #ifdef __cplusplus
 };
 #endif
